///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008, Industrial Light & Magic, a division of Lucas
// Digital Ltd. LLC
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// *       Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// *       Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
// *       Neither the name of Industrial Light & Magic nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////


//-----------------------------------------------------------------------------
//
//      class CfaPixel, class CfaTile
//
//-----------------------------------------------------------------------------

#include <ImfCfaTile.h>


namespace Imf {


CfaPixel::CfaPixel ():
    channelName(),
    xOffset (0),
    yOffset (0),
    XYZ (0, 0, 0)
{
    // empty
}


CfaPixel::CfaPixel
    (const char channelName[],
     int xOffset,
     int yOffset,
     const Imath::V3f &XYZ)
:
    channelName (channelName),
    xOffset (xOffset),
    yOffset (yOffset),
    XYZ (XYZ)
{
    // empty
}


CfaTile::CfaTile (int xSize, int ySize):
    _xSize (xSize),
    _ySize (ySize),
    _pixels (new CfaPixel[xSize * ySize])
{
    // empty
}


CfaTile::CfaTile (const CfaTile &other):
    _xSize (other._xSize),
    _ySize (other._ySize),
    _pixels (new CfaPixel[_xSize * _ySize])
{
    for (size_t i = 0; i < _xSize * _ySize; ++i)
        _pixels[i] = other._pixels[i];
}


CfaTile::~CfaTile ()
{
    delete [] _pixels;
}


CfaTile &          
CfaTile::operator = (const CfaTile &other)
{
    CfaPixel *newPixels = new CfaPixel[other._xSize * other._ySize];

    for (size_t i = 0; i < other._xSize * other._ySize; ++i)
        newPixels[i] = other._pixels[i];

    _xSize = other._xSize;
    _ySize = other._ySize;
    _pixels = newPixels;

    return *this;
}


void               
CfaTile::resize (int newXSize, int newYSize)
{
    CfaPixel *newPixels = new CfaPixel[newXSize * newYSize];

    int xSizeCopy = std::min (_xSize, newXSize);
    int ySizeCopy = std::min (_ySize, newYSize);

    for (int y = 0; y < ySizeCopy; ++y)
        for (int x = 0; x < xSizeCopy; ++x)
            newPixels[x + y * newXSize] = _pixels[x + y * _xSize];

    delete [] _pixels;

    _xSize = newXSize;
    _ySize = newYSize;
    _pixels = newPixels;
}

} // namespace Imf
