///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008, Industrial Light & Magic, a division of Lucas
// Digital Ltd. LLC
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// *       Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// *       Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
// *       Neither the name of Industrial Light & Magic nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////


//-----------------------------------------------------------------------------
//
//      class CfaTileAttribute
//
//-----------------------------------------------------------------------------

#include <ImfCfaTileAttribute.h>


namespace Imf {


template <>
const char *
CfaTileAttribute::staticTypeName ()
{
    return "cfatile";
}


template <>
void
CfaTileAttribute::writeValueTo (OStream &os, int version) const
{
    Xdr::write <StreamIO> (os, _value.xSize());
    Xdr::write <StreamIO> (os, _value.ySize());

    for (int y = 0; y < _value.ySize(); ++y)
    {
        for (int x = 0; x < _value.xSize(); ++x)
        {
            const CfaPixel &pixel = _value.pixel (x, y);

            Xdr::write <StreamIO> (os, pixel.channelName.text());
            Xdr::write <StreamIO> (os, pixel.xOffset);
            Xdr::write <StreamIO> (os, pixel.yOffset);
            Xdr::write <StreamIO> (os, pixel.XYZ.x);
            Xdr::write <StreamIO> (os, pixel.XYZ.y);
            Xdr::write <StreamIO> (os, pixel.XYZ.z);
            Xdr::pad   <StreamIO> (os, 16);
        }
    }
}


template <>
void
CfaTileAttribute::readValueFrom (IStream &is, int size, int version)
{
    int xSize, ySize;
    Xdr::read <StreamIO> (is, xSize);
    Xdr::read <StreamIO> (is, ySize);

    _value.resize (xSize, ySize);

    for (int y = 0; y < _value.ySize(); ++y)
    {
        for (int x = 0; x < _value.xSize(); ++x)
        {
            CfaPixel &pixel = _value.pixel (x, y);

            char channelName[Name::SIZE];
            Xdr::read <StreamIO> (is, sizeof (channelName), channelName);
            pixel.channelName = channelName;

            Xdr::read <StreamIO> (is, pixel.xOffset);
            Xdr::read <StreamIO> (is, pixel.yOffset);
            Xdr::read <StreamIO> (is, pixel.XYZ.x);
            Xdr::read <StreamIO> (is, pixel.XYZ.y);
            Xdr::read <StreamIO> (is, pixel.XYZ.z);
            Xdr::skip <StreamIO> (is, 16);
        }
    }
}


} // namespace Imf
