#include <interpolatePixels.h>


using namespace Imf;


void
interpolatePixels (Array2D <half> &pixels, int w, int h)
{
    Array2D <half> xPixels (h, w);
    Array2D <half> yPixels (h, w);
    bool nansLeftInPixels = true;

    while (nansLeftInPixels)
    {
        //
        // Horizontal pass
        //

        for (int y = 0; y < h; ++y)
        {
            bool rowIsAllNans = true;

            for (int x = 0; x < w; ++x)
            {
                xPixels[y][x] = half::qNan();

                if (!pixels[y][x].isNan())
                    rowIsAllNans = false;
            }

            if (rowIsAllNans)
                continue;

            for (int x = 0; x < w; ++x)
            {
                if (pixels[y][x].isNan())
                {
                    int x1 = x - 1;

                    while (pixels[y][abs (x1)].isNan())
                        --x1;

                    int x2 = x + 1;

                    while (pixels[y][(w - 1) - abs (w - 1 - x2)].isNan())
                        ++x2;

                    float s = float (x - x1) / (x2 - x1);
                    float t = float (x2 - x) / (x2 - x1);

                    xPixels[y][x] = s * pixels[y][abs (x1)] +
                                    t * pixels[y][(w - 1) - abs (w - 1 - x2)];
                }
                else
                {
                    xPixels[y][x] = pixels[y][x];
                }
            }
        }

        //
        // Vertical pass
        //

        for (int x = 0; x < w; ++x)
        {
            bool columnIsAllNans = true;

            for (int y = 0; y < h; ++y)
            {
                yPixels[y][x] = half::qNan();

                if (!pixels[y][x].isNan())
                    columnIsAllNans = false;
            }

            if (columnIsAllNans)
                continue;

            for (int y = 0; y < h; ++y)
            {
                if (pixels[y][x].isNan())
                {
                    int y1 = y - 1;

                    while (pixels[abs (y1)][x].isNan())
                        --y1;

                    int y2 = y + 1;

                    while (pixels[(h - 1) - abs (h - 1 - y2)][x].isNan())
                        ++y2;

                    float s = float (y - y1) / (y2 - y1);
                    float t = float (y2 - y) / (y2 - y1);

                    yPixels[y][x] = s * pixels[abs (y1)][x] +
                                    t * pixels[(h - 1) - abs (h - 1 - y2)][x];
                }
                else
                {
                    yPixels[y][x] = pixels[y][x];
                }
            }
        }

        //
        // Combine both passes
        //

        nansLeftInPixels = false;

        for (int y = 0; y < h; ++y)
        {
            for (int x = 0; x < w; ++x)
            {
                if (xPixels[y][x].isNan())
                {
                    if (yPixels[y][x].isNan())
                        nansLeftInPixels = true;
                    else
                        pixels[y][x] = yPixels[y][x];
                }
                else
                {
                    if (yPixels[y][x].isNan())
                        pixels[y][x] = xPixels[y][x];
                    else
                        pixels[y][x] = (xPixels[y][x] + yPixels[y][x]) / 2;
                }
            }
        }
    }
}
