#include <rgbToCfaRgb.h>
#include <cfaToRgb.h>

#include <iostream>
#include <exception>
#include <string>
#include <stdlib.h>

using namespace std;


void
usageMessage (const char argv0[], bool verbose = false)
{
    cerr << "usage:\n"
            "   " << argv0 << "[options] -toRgb rgb_cfa_file rgb_file\n"
            "   " << argv0 << "[options] -toCfaRgb rgb_file rgb_cfa_file\n" <<
            endl;

    if (verbose)
    {
	cerr << "\n"
                "-toRgb     Converts a raw image that was captured with a\n"
                "           color filter array sensor into a regular RGB\n"
                "           image.\n"
                "\n"
                "-toCfaRgb  Converts an RGB image into an image that\n"
                "           simulates the output from a Bayer-pattern\n"
                "           color filter array sensor.\n"
		"\n"
		"-v         verbose mode\n"
		"\n"
		"-h         prints this message\n";

	 cerr << endl;
    }

    exit (1);
}


int
main(int argc, char **argv)
{
    const char *inFile = 0;
    const char *outFile = 0;
    bool toRgb = false;
    bool toCfaRgb = false;
    bool verbose = false;

    //
    // Parse the command line.
    //

    if (argc < 3)
	usageMessage (argv[0], true);

    int i = 1;

    while (i < argc)
    {
	if (!strcmp (argv[i], "-toRgb"))
	{
	    //
	    // Convert from CFA to RGB
	    //

	    toRgb = true;
	    i += 1;
	}
	else if (!strcmp (argv[i], "-toCfaRgb"))
	{
	    //
	    // Convert from RGB to RGB CFA
	    //

	    toCfaRgb = true;
	    i += 1;
	}
	else if (!strcmp (argv[i], "-v"))
	{
	    //
	    // Verbose mode
	    //

	    verbose = true;
	    i += 1;
	}
	else if (!strcmp (argv[i], "-h"))
	{
	    //
	    // Print help message
	    //

	    usageMessage (argv[0], true);
	}
	else
	{
	    //
	    // Image file name
	    //

	    if (inFile == 0)
		inFile = argv[i];
	    else
		outFile = argv[i];

	    i += 1;
	}
    }

    if (inFile == 0 || outFile == 0 ||
        int (toRgb) + int (toCfaRgb) != 1)
    {
	usageMessage (argv[0]);
    }

    //
    // Do the work
    //

    int exitStatus = 0;

    try
    {
        if (toRgb)
            cfaToRgb (inFile, outFile, verbose);

        if (toCfaRgb)
            rgbToCfaRgb (inFile, outFile, verbose);
    }
    catch (const exception &e)
    {
	cerr << e.what() << endl;
	exitStatus = 1;
    }

    return exitStatus;
}
